#
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the Common Development
# and Distribution License Version 1.0 (the "License").
#
# You can obtain a copy of the license at
# http://www.opensource.org/licenses/CDDL-1.0.  See the License for the
# specific language governing permissions and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each file and
# include the License file in a prominent location with the name LICENSE.CDDL.
# If applicable, add the following below this CDDL HEADER, with the fields
# enclosed by brackets "[]" replaced with your own identifying information:
#
# Portions Copyright (c) [yyyy] [name of copyright owner]. All rights reserved.
#
# CDDL HEADER END
#

#
# Copyright (c) 2018--2019, Regents of the University of Minnesota.
# All rights reserved.
#
# Contributors:
#    Ryan S. Elliott
#

#
# Release: This file is part of the kim-api-2.1.3 package.
#

_kim_api_collections_management()
{
  local collections_info
  collections_info=/usr/libexec/kim-api/kim-api-collections-info

  local cd_type
  if test -n "${ZSH_VERSION}"; then
    cd_type=`type -w _cd | sed -e 's/^[^:]*: *//'`
  else
    cd_type=`type -t _cd`
  fi

  local cur subcommands opts
  subcommands="list set-user-model-drivers-dir set-user-portable-models-dir "
  subcommands="${subcommands} set-user-simulator-models-dir install reinstall"
  subcommands="${subcommands} remove remove-all"

  COMPREPLY=()
  cur="${COMP_WORDS[COMP_CWORD]}"

  if test ${COMP_CWORD} -eq 1; then
    COMPREPLY=( $(compgen -W "${subcommands}" -- "${cur}") )
    return 0
  else
    case "${COMP_WORDS[1]}" in
      list)
        opts=""
        if test ${COMP_CWORD} -eq 2; then
          opts="--log"
        else
          return 1
        fi
        COMPREPLY=( $(compgen -W "${opts}" -- "${cur}") )
        return 0
        ;;
      set-user-*)
        if test ${COMP_CWORD} -gt 2; then
          return 1
        else
          if test x"function" = x"${cd_type}"; then  # check if bash_completion has been loaded & _cd defined
            _cd
          fi
          return 0
        fi
        ;;
      install)
        if test ${COMP_CWORD} -eq 2; then
          COMPREPLY=( $(compgen -W "--force CWD environment user system" \
                                -- "${cur}") )
          return 0
        fi
        if test ${COMP_CWORD} -eq 3 -a x"${COMP_WORDS[2]}" = x"--force"; then
          COMPREPLY=( $(compgen -W "CWD environment user system" \
                                -- "${cur}") )
          return 0
        fi
        local collection_position
        if test x"${COMP_WORDS[2]}" = x"--force"; then
          collection_position=3
        else
          collection_position=2
        fi
        opts=""
        if test ${COMP_CWORD} -eq `expr ${collection_position} + 1` \
                -a x"system" = x"${COMP_WORDS[${collection_position}]}"; then
          opts="--sudo OpenKIM"
        elif test ${COMP_CWORD} -eq 3; then
          opts="OpenKIM"
        fi

        local query='query={"type":{"$regex":"md|mo|sm"},"kim-api-version":{"$regex":"^2\\."}}'
        query="${query}"'&fields={"kimcode":1}'
        query="${query}"'&database=obj&history=on'
        local list=`wget -q -O - --post-data="${query}" \
                    https://query.openkim.org/api | \
                    sed -e 's/\[//g' -e 's/\]//g' \
                    -e 's/{"kimcode": "\([^"]*\)"},*/\1/g'`
        opts="${opts} ${list}"

        if test x"function" = x"cd_type"; then  # check if bash_completion has been loaded & _cd defined
          _cd
        fi
        COMPREPLY=( "${COMPREPLY[@]}" $(compgen -W "${opts}" -- "${cur}") )
        return 0
        ;;
      reinstall|remove)
        opts=""
        if test ${COMP_CWORD} -eq 2; then
          opts="--force --interactive --sudo"
        elif test ${COMP_CWORD} -eq 3 -a x"--interactive" = x"${COMP_WORDS[2]}"; then
          opts="--sudo"
        elif test ${COMP_CWORD} -eq 3 -a x"--force" = x"${COMP_WORDS[2]}"; then
          opts="--sudo"
        fi
        local models drivers simulator_models
        drivers=`${collections_info} model_drivers | sed -e 's|[^[:space:]]* \([^[:space:]]*\) .*|\1|'`
        models=`${collections_info} portable_models | sed -e 's|[^[:space:]]* \([^[:space:]]*\) .*|\1|'`
        drivers=`${collections_info} simulator_models | sed -e 's|[^[:space:]]* \([^[:space:]]*\) .*|\1|'`
        opts="${opts} ${drivers} ${models} ${simulator_models}"

        if test x"reinstall" = x"${COMP_WORDS[1]}"; then
          if test x"function" = x"cd_type"; then  # check if bash_completion has been loaded & _cd defined
            _cd
          fi
        fi
        COMPREPLY=( "${COMPREPLY[@]}" $(compgen -W "${opts}" -- "${cur}") )
        return 0
        ;;
      remove-all)
        opts=""
        if test ${COMP_CWORD} -eq 2; then
          opts="--force --interactive --sudo"
        elif test ${COMP_CWORD} -eq 3 -a x"--interactive" = x"${COMP_WORDS[2]}"; then
          opts="--sudo"
        elif test ${COMP_CWORD} -eq 3 -a x"--force" = x"${COMP_WORDS[2]}"; then
          opts="--sudo"
        fi
        COMPREPLY=( $(compgen -W "${opts}" -- "${cur}") )
        return 0
        ;;
      *)
        return 1
        ;;
    esac
  fi
  return 1
}
complete -F _kim_api_collections_management kim-api-collections-management
